@@ -1,5 +1,5 @@
 # GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005 Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006 Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,6 +39,12 @@ class NoQuestionInFileException(LessonfileException):
     def __str__(self):
         return "The lesson file '%s' contains no questions." % self.m_filename
 
+class FileNotFound(LessonfileException):
+    def __init__(self, filename):
+        self.m_filename = filename
+    def __str__(self):
+        return "The external file '%s' was not found." % self.m_filename
+
 class NoQuestionConfiguredException(LessonfileException):
     """
     This exception is raised by select_random_question if the user has
@@ -58,6 +64,9 @@ import soundcard
 import dataparser
 import osutils
 
+class plstring(str):
+    pass
+
 # The keys in the dict say how many steps up or down in the
 # circle of fifths we go if we transpose.
 _keys_to_interval = {
@@ -102,6 +111,7 @@ keywords = (
     'twelvetone',
     'identifybpm',
     'nameinterval',
+    'elembuilder',
     #
     'harmonic',
     'melodic',
@@ -112,6 +122,7 @@ keywords = (
     'satb',
     'cmdline',
     'wavfile',
+    'midifile',
     'horiz',
     'vertic',
     'accidentals',
@@ -119,6 +130,8 @@ keywords = (
     'semitones',
     'play',
     'show',
+    # elembuilder:
+    'auto',
     # Used by the rhythm module:
     'newline',
     'd1', 'p1', 'a1',
@@ -143,7 +156,9 @@ for n in keywords:
 lessonfile_functions = {
         '_': dataparser.dataparser_i18n_func,
         '_i': _i, #FIXME
+        # play_wav should probably be removed. Replaced by wavfile
         'play_wav': lambda f: Music(cfg.get_string("sound/wav_player") + " " + f, "cmdline"),
+        # Update Music.is_displayable when adding music types
         'music': lambda m, t='normal': Music(m, t),
         'chord': lambda m: Music(m, 'chord'),
         'satb': lambda m: Music(m, 'satb'),
@@ -151,6 +166,8 @@ lessonfile_functions = {
         'rvoice': lambda m: Music(m, 'rvoice'),
         'cmdline': lambda m: Music(m, 'cmdline'),
         'wavfile': lambda m: Music(m, 'wavfile'),
+        'midifile': lambda m: Music(m, 'midifile'),
+        'progressionlabel': lambda s: plstring(s),
 }
 
 class _Header(dict):
@@ -194,6 +211,9 @@ class Music:
     def __init__(self, musicdata, musictype='normal'):
         self.m_musicdata = musicdata
         self.m_musictype = musictype
+    def is_displayable(self):
+        return self.m_musictype in ('normal', 'chord',
+                'satb', 'voice', 'rvoice')
 
 def parse_test_def(s):
     m = re.match("(\d+)\s*x", s)
@@ -210,13 +230,15 @@ class LessonfileCommon(object):
         self.header     a Header instance
         self.questions  a list of all question
         """
-        parser = dataparser.Dataparser(predef, lessonfile_functions, ('tempo',))
+        self.dataparser = dataparser.Dataparser(predef, lessonfile_functions, ('tempo',))
         self.m_filename = filename
-        parser.parse_file(filename)
+        self.dataparser.parse_file(filename)
         self.m_transpose = mpd.MusicalPitch.new_from_notename("c'")
-        self.header = _Header(parser.header)
-        self.m_globals = parser.globals
-        self.m_questions = parser.questions
+        self.header = _Header(self.dataparser.header)
+        self.m_globals = self.dataparser.globals
+        self.m_questions = self.dataparser.questions
+        self.blocklists = self.dataparser.blocklists
+        del self.dataparser
         for question in self.m_questions:
             question['active'] = 1
             # FIXMECOMPAT
@@ -232,18 +254,13 @@ class LessonfileCommon(object):
         if self.header.random_transpose and len(self.header.random_transpose) == 2:
             self.header.random_transpose \
                 = ['semitones'] + self.header.random_transpose
-    def generate_test_questions(self):
-        count, t = parse_test_def(self.header.test)
-        q = range(len(self.m_questions)) * count
-        random.shuffle(q)
-        return q
-    def get_test_requirement(self):
-        """
-        Return the amount of exercises that has to be correct to
-        pass the test. (values 0.0 to 1.0)
-        """
-        m = re.match("([\d\.]+)%", self.header.test_requirement)
-        return float(m.groups()[0])/100.0
+        # Some variables does only make sense if we have a music displayer
+        if self.header.at_question_start:
+            self.header.have_music_displayer = True
+
+class QuestionsLessonfile(LessonfileCommon):
+    def __init__(self, teacher):
+        LessonfileCommon.__init__(self, teacher)
     def select_random_question(self):
         """
         Select a new question by random. It will use the music in the
@@ -445,7 +462,7 @@ class LessonfileCommon(object):
                  rel_note, " ".join(v))
         elif question[varname].m_musictype == 'cmdline':
             return str(question[varname])
-        elif question[varname].m_musictype == 'wavfile':
+        elif question[varname].m_musictype in ('wavfile', 'midifile'):
             return question[varname].m_musicdata
         else:
             raise LessonfileException("\nUknown music type: '%s'" % question[varname].m_musictype)
@@ -510,7 +527,6 @@ class LessonfileCommon(object):
             return v
         else:
             raise LessonfileException("get_music_as_notename_list cannot handle m_musictype==%s" % music.m_musictype)
- 
     def get_music_as_notename_list(self, varname):
         """
         Return a list of notenames from the variabale VARNAME in the
@@ -535,25 +551,6 @@ class LessonfileCommon(object):
         Return True if a question is selected.
         """
         return self._idx is not None
-    def enter_test_mode(self):
-        self.m_test_questions = self.generate_test_questions()
-        self.m_test_idx = -1
-    def next_test_question(self):
-        assert self.m_test_idx < len(self.m_test_questions)
-        self.m_test_idx += 1
-        self._idx = self.m_test_questions[self.m_test_idx]
-        if self.header.random_transpose:
-            old = self.m_transpose
-            # try really hard not to get the same tonika:
-            for x in range(100):
-                self.m_transpose = self.find_random_transpose()
-                if old != self.m_transpose:
-                    break
-    def is_test_complete(self):
-        """
-        Return True if the test is compleded.
-        """
-        return self.m_test_idx == len(self.m_test_questions) -1
     def play_question(self, question=None, varname='music'):
         """Play the question. Play the current question if question is none.
         varname is the name of the variable that contains the music.
@@ -563,7 +560,16 @@ class LessonfileCommon(object):
         if question[varname].m_musictype == 'cmdline':
             osutils.run_external_program(str(question[varname].m_musicdata), os.path.split(self.m_t.m_lessonfile)[0], "")
         elif question[varname].m_musictype == 'wavfile':
-            soundcard.play_wav_file(os.path.join(
+            if os.path.exists(os.path.join(
+                        os.path.split(self.m_t.m_lessonfile)[0],
+                        question[varname].m_musicdata)):
+                    soundcard.play_wav_file(os.path.join(
+                        os.path.split(self.m_t.m_lessonfile)[0],
+                        question[varname].m_musicdata))
+            else:
+                raise FileNotFound(question[varname].m_musicdata)
+        elif question[varname].m_musictype == 'midifile':
+            soundcard.play_midi_file(os.path.join(
                 os.path.split(self.m_t.m_lessonfile)[0],
                 question[varname].m_musicdata))
         elif question[varname].m_musictype in ('normal', 'satb', 'voice', 'rvoice'):
@@ -742,11 +748,56 @@ class LessonfileCommon(object):
                     instrument[x+1] = cfg.get_int('config/preferred_instrument_velocity')
         return instrument
 
+class TestSupport(object):
+    """
+    Lessonfile classes can add this class to the list of classes it
+    inherits from if the exercise want to have tests.
+    """
+    def _generate_test_questions(self):
+        count, t = parse_test_def(self.header.test)
+        q = range(len(self.m_questions)) * count
+        random.shuffle(q)
+        return q
+    def get_test_requirement(self):
+        """
+        Return the amount of exercises that has to be correct to
+        pass the test. (values 0.0 to 1.0)
+        """
+        m = re.match("([\d\.]+)%", self.header.test_requirement)
+        return float(m.groups()[0])/100.0
+    def enter_test_mode(self):
+        self.m_test_questions = self._generate_test_questions()
+        self.m_test_idx = -1
+    def next_test_question(self):
+        assert self.m_test_idx < len(self.m_test_questions)
+        self.m_test_idx += 1
+        self._idx = self.m_test_questions[self.m_test_idx]
+        if self.header.random_transpose:
+            old = self.m_transpose
+            # try really hard not to get the same tonika:
+            for x in range(100):
+                self.m_transpose = self.find_random_transpose()
+                if old != self.m_transpose:
+                    break
+    def is_test_complete(self):
+        """
+        Return True if the test is compleded.
+        """
+        return self.m_test_idx == len(self.m_test_questions) -1
 
-class DictationLessonfile(LessonfileCommon):
+class HeaderLessonfile(LessonfileCommon):
+    """
+    This lesson file class should be used by all the exercise modules
+    that does not need any question blocks defined.
+    """
     def __init__(self, filename, teacher):
         LessonfileCommon.__init__(self, teacher)
         self.parse_file(filename)
+
+class DictationLessonfile(QuestionsLessonfile):
+    def __init__(self, filename, teacher):
+        QuestionsLessonfile.__init__(self, teacher)
+        self.parse_file(filename)
     def get_breakpoints(self):
         assert self._idx is not None
         r = []
@@ -785,15 +836,14 @@ class DictationLessonfile(LessonfileCommon):
         """
         self._idx = 0
 
-class SingChordLessonfile(LessonfileCommon):
+class SingChordLessonfile(QuestionsLessonfile):
     def __init__(self, filename, teacher):
-        LessonfileCommon.__init__(self, teacher)
+        QuestionsLessonfile.__init__(self, teacher)
         self.parse_file(filename)
 
-class NameIntervalLessonfile(LessonfileCommon):
+class NameIntervalLessonfile(HeaderLessonfile):
     def __init__(self, filename, teacher):
-        LessonfileCommon.__init__(self, teacher)
-        self.parse_file(filename)
+        HeaderLessonfile.__init__(self, filename, teacher)
         iquality = []
         inumbers = []
         self.header.intervals = [mpd.Interval(n) for n in self.header.intervals]
@@ -827,18 +877,16 @@ class NameIntervalLessonfile(LessonfileCommon):
                 raise LessonfileException("Error in the lesson file '%s':\nThe length of self.header.tones is not 2" % self.m_filename)
             self.header.tones = [mpd.MusicalPitch.new_from_notename(n) for n in self.header.tones]
 
-class IdByNameLessonfile(LessonfileCommon):
+class IdByNameLessonfile(QuestionsLessonfile, TestSupport):
     def __init__(self, filename, teacher):
         """
         teacher -- the teacher class contructing this lesson file class
         """
-        LessonfileCommon.__init__(self, teacher)
+        QuestionsLessonfile.__init__(self, teacher)
+        TestSupport.__init__(self)
         self.parse_file(filename)
     def parse_file(self, filename):
         super(IdByNameLessonfile, self).parse_file(filename)
-        # Some variables does only make sense if we have a music displayer
-        if self.header.at_question_start:
-            self.header.have_music_displayer = True
         # Also, if some questions has cuemusic, then we need the displayer
         if [q for q in self.m_questions if 'cuemusic' in q]:
             self.header.have_music_displayer = True
@@ -852,21 +900,40 @@ class IdByNameLessonfile(LessonfileCommon):
             else:
                 self.m_questions.append(question)
 
-class SingAnswerLessonfile(LessonfileCommon):
+class SingAnswerLessonfile(QuestionsLessonfile):
     def __init__(self, filename, teacher):
         """
         teacher -- the teacher class contructing this lesson file class
         """
-        LessonfileCommon.__init__(self, teacher)
+        QuestionsLessonfile.__init__(self, teacher)
         self.parse_file(filename)
     def parse_file(self, filename):
-        # FIXME this method is only necessary if we to something in it.
         super(SingAnswerLessonfile, self).parse_file(filename)
+        v = [q for q in self.m_questions if 'question_text' not in q]
+        if [q for q in self.m_questions if 'question_text' not in q]:
+            raise LessonfileException(_('Question number %(index)i in the lesson file "%(filename)s" is missing the "question_text" variable.') % {
+                'index': self.m_questions.index(v[0]),
+                'filename': filename})
 
+class IntervalsLessonfile(HeaderLessonfile, TestSupport):
+    """
+    Common lesson file class for some interval exercises.
+    We inherit from TestSupport, but overwrites some methods from it.
+    """
+    def enter_test_mode(self):
+        count, t = parse_test_def(self.header.test)
+        if self.header.intervals:
+            self.m_test_questions = self.header.intervals * count
+        else:
+            self.m_test_questions = self.header.ask_for_intervals_0 * count
+        random.shuffle(self.m_test_questions)
+        self.m_test_idx = -1
+    def next_test_question(self):
+        self.m_test_idx += 1
 
-class ChordLessonfile(LessonfileCommon):
+class ChordLessonfile(QuestionsLessonfile):
     def __init__(self, filename, teacher):
-        LessonfileCommon.__init__(self, teacher)
+        QuestionsLessonfile.__init__(self, teacher)
         self.parse_file(filename)
     def parse_file(self, filename):
         """
@@ -910,12 +977,22 @@ class ChordLessonfile(LessonfileCommon):
             return self.m_questions[self._idx]['toptone']
         return -1
 
+class ElembuilderLessonfile(QuestionsLessonfile):
+    def __init__(self, filename, teacher):
+        QuestionsLessonfile.__init__(self, teacher)
+        self.parse_file(filename)
+    def parse_file(self, filename):
+        super(ElembuilderLessonfile, self).parse_file(filename)
+
+
 class LessonFileManager:
-    def __init__(self):
-        self.parse()
-    def parse(self):
+    def __init__(self, debug):
+        self.parse(debug)
+    def parse(self, debug):
         lessonpath = ['lesson-files',
                       os.path.join(gethomedir.get_home_dir(), 'lessonfiles')]
+        if debug:
+            lessonpath.append('regression-lesson-files')
         self.m_uiddb = {}
         vim_tmpfile_re = re.compile("\..*\.swp")
         for dir in lessonpath:
@@ -1114,14 +1191,13 @@ if __name__ == '__main__':
         "~/.solfegerc")
     t2 = time.clock()
     for n in range(1):
-        #p = LessonfileCommon()
-        #p.parse_file(sys.argv[1])
+        p = ElembuilderLessonfile(sys.argv[1], None)
         #p = LessonFileManager()
-        p = LessonfileCommon(None)
-        p.parse_file('lesson-files/chord-min-major')
-        print p.m_globals
+        #p = LessonfileCommon(None)
+        #p.parse_file('lesson-files/chord-min-major')
         for q in p.m_questions:
             print q
+        print p.blocklists['element']
     t3 = time.clock()
     #print t3-t2
     #print t2-t1
