@@ -1,7 +1,7 @@
 # -*- coding: iso-8859-1 -*-
 
 # GNU Solfege - ear training for GNOME
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006  Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin ST, Fifth Floor, Boston, MA  02110-1301  USA
 
-copyright = "Copyright  1999-2006 Tom Cato Amundsen <tca@gnu.org>, and others"
+copyright = "Copyright  1999-2006 Tom Cato Amundsen <tca@gnu.org>, and others."
 
 import sys
 import os
@@ -27,6 +27,8 @@ import urllib
 import runtime
 
 import configureoutput
+import src
+import osutils
 
 import optparse
 
@@ -53,14 +55,16 @@ opt_parser.add_option('--verbose-sound-init', action='store_true',
 opt_parser.add_option('--no-sound', action='store_true', dest='no_sound',
     default=False,
     help=_('Do not play any sounds. Instead some data is printed to standard output. Use this for debugging and porting.'))
+opt_parser.add_option('--debug', action='store_true', dest='debug',
+    help=_('Include features used by the Solfege developers to debug the program.'))
+opt_parser.add_option('--disable-exception-handler', action='store_true',
+    dest='disable_exception_handler',
+    help=_("Disable the exception handling in Gui.standard_exception_handler."))
 opt_parser.add_option('--no-random', action='store_true', dest='no_random',
     help=_('For debugging only: Select questions from lesson files in sequential order.'))
 opt_parser.add_option('--without-gtkhtml', action='store_true', 
     dest='without_gtkhtml',
     help=_('Run without using gtkhtml2 even if it is installed.'))
-opt_parser.add_option('--without-gnome', action='store_true', 
-    dest='without_gnome',
-    help=_('Run without the gnome module (especially gnomecanvas) even if it is installed.'))
 opt_parser.add_option('--no-cairo-widgets', action='store_true',
     dest='no_cairo_widgets',
     help=_("Do not use the cairo version of input widgets, even if we run gtk+ 2.8.0 or newer."))
@@ -101,14 +105,11 @@ USA.
     sys.exit()
 
 # silent, GNOME, be silent!
-sys.argv.append('--disable-sound')
+#sys.argv.append('--disable-sound')
 runtime.init(options)
 
 import gtk
 
-if not runtime.has_gnome():
-    sys.path.insert(0, "gnomeemu")
-
 class SplashWin(gtk.Window):
     def __init__(self):
         gtk.Window.__init__(self, gtk.WINDOW_POPUP)
@@ -142,17 +143,6 @@ if not options.no_splash:
 else:
     splash_win = None
 
-if splash_win:
-    splash_win.show_progress("importing GNOME modules")
-
-import gnome, gnome.ui
-# FIXME PAD AND PAD_SMALL are missing from some gnome-python releases.
-try:
-    gnome.ui.PAD
-except:
-    gnome.ui.PAD = 8
-    gnome.ui.PAD_SMALL = 4
-
 if splash_win:
     splash_win.show_progress("importing application modules")
 
@@ -194,7 +184,8 @@ import rhythm
 import identifybpm
 import harmonicprogressiondictation
 import singanswer
-#import example
+import elembuilder
+
 import learning_tree_editor
 from docviewer import DocViewer
 from helpbrowser import HelpBrowser
@@ -215,10 +206,55 @@ class MusicViewerWindow(gtk.Dialog):
         fontsize = cfg.get_int('config/feta_font_size=20')
         self.g_music_displayer.display(music, fontsize)
 
-class MainWin(gnome.ui.App, cfg.ConfigUtils):
+class App(gtk.Window):
+    def __init__(self, a, b):
+        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
+        self._cid = 4
+        self.__vbox = gtk.VBox()
+        self.add(self.__vbox)
+        self.__menubox = gu.bHBox(self.__vbox, False)
+        self.__toolbarbox = gu.bHBox(self.__vbox, False)
+        self.__contentsbox = gu.bVBox(self.__vbox)
+        self.__appbarbox = gu.bHBox(self.__vbox, False)
+        self.__dockitems = {}
+    def show(self):
+        self.__vbox.show()
+        self.__menubox.show()
+        self.__toolbarbox.show()
+        self.__contentsbox.show()
+        self.__appbarbox.show()
+        gtk.Window.show(self)
+    def set_statusbar(self, bar):
+        self.__appbarbox.pack_start(bar)
+        bar.show()
+        self.__appbar = bar
+    def set_contents(self, c):
+        self.__contentsbox.pack_start(c)
+    def set_menus(self, menu):
+        hdlbox = gtk.HandleBox()
+        hdlbox.show()
+        self.__dockitems['Menubar'] = hdlbox
+
+        self.__menubox.pack_start(hdlbox, True, True)
+        hdlbox.add(menu)
+        menu.show()
+    def add_toolbar(self, toolbar, name, behavior, placement, band_num,
+                    band_position, offset):
+        hdlbox = gtk.HandleBox()
+        hdlbox.show()
+        self.__dockitems[name] = hdlbox
+        self.__toolbarbox.pack_start(hdlbox)
+        self.__toolbarbox.reorder_child(hdlbox, offset)
+        hdlbox.add(toolbar)
+        toolbar.show()
+    def get_dock_item_by_name(self, name):
+        return self.__dockitems[name]
+
+class MainWin(App, cfg.ConfigUtils):
     default_learning_tree = ('solfege', 'learningtree.txt')
+    debug_learning_tree = ('solfege', 'debugtree.txt')
     def __init__(self, options, datadir, lessonfile_manager):
-        gnome.ui.App.__init__(self, 'solfege', 'GNU Solfege')
+        App.__init__(self, 'solfege', 'GNU Solfege')
         self.icons = stock.SolfegeIconFactory(self, datadir)
         self.set_default_size(400, 400)
         pixbuf = self.render_icon('solfege-icon', gtk.ICON_SIZE_DIALOG)
@@ -266,6 +302,9 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
             loc, filename = self.get_list("app/learningtree")
         except ValueError:
             loc, filename = self.default_learning_tree
+        if (loc, filename) == ('solfege', 'debugtree.txt') and not self.m_app.m_options.debug:
+            self.set_list("app/learningtree", ('solfege', 'learningtree.txt'))
+            loc, filename = self.default_learning_tree
         try:
             if loc == 'solfege':
                 self.m_tree.load(filename)
@@ -337,6 +376,12 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
             None, None,
            lambda o: self.change_learning_tree(self.default_learning_tree)
             )]
+        if self.m_app.m_options.debug:
+            actions = [('LearningTreesMenu', None, _('Learning tree')),
+                ('DefaultTree', None, _("Debugging"),
+                None, None,
+               lambda o: self.change_learning_tree(self.debug_learning_tree)
+                )]
         s = """
 <menubar name='Menubar'>
  <menu action='FileMenu'>
@@ -450,27 +495,42 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
                 })
             import webbrowser
             webbrowser.open_new("http://www.solfege.org?%s" % params)
-    def display_exception_message(self, exception, err_file=None, msg=None):
+    def display_exception_message(self, exception, err_file=None, msg=""):
         """Call this function only inside an except clause."""
         if err_file:
-            msg = _("Exception caught in %s:%i\n") % (err_file, sys.exc_info()[2].tb_lineno)
-        elif not msg:
-            msg = ""
+            msg += _('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': err_file, 'lineno': sys.exc_info()[2].tb_lineno}
         exception_str = unicode(exception)
         if not isinstance(exception_str, unicode):
             exception_str = exception_str.decode(locale.getpreferredencoding(), 'replace')
-        msg = "\n".join([msg, exception_str])
         m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
-                              gtk.BUTTONS_CLOSE, msg)
+                              gtk.BUTTONS_CLOSE, str(exception))
         m.set_title(sys.exc_info()[0].__name__)
+        m.format_secondary_text(msg)
         m.run()
         m.destroy()
-    def display_error_message(self, msg):
+    def display_error_message2(self, text, secondary_text):
+        """
+        This is the new version of display_error_message, and it will
+        eventually replace the old.
+        """
+        if not isinstance(text, unicode):
+            msg = msg.decode(locale.getpreferredencoding(), 'replace')
+        m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
+                              gtk.BUTTONS_CLOSE, text)
+        if secondary_text:
+            m.format_secondary_text(secondary_text)
+        m.run()
+        m.destroy()
+    def display_error_message(self, msg, title=None, secondary_text=None):
         if not isinstance(msg, unicode):
             msg = msg.decode(locale.getpreferredencoding(), 'replace')
         m = gtk.MessageDialog(self, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
                               gtk.BUTTONS_CLOSE, None)
         m.set_markup(msg)
+        if title:
+            m.set_title(title)
+        if secondary_text:
+            m.format_secondary_text(secondary_text)
         m.run()
         m.destroy()
     def display_question_music_error_message(self, idx, lessonfile, exception, err_file=None):
@@ -478,40 +538,44 @@ class MainWin(gnome.ui.App, cfg.ConfigUtils):
         msg = _("Failed to parse the music for question number %(idx)i in '%(lf)s':\n%(ex)s") % {'idx': idx, 'lf': lessonfile, 'ex': exception}
         self.display_exception_message(msg, err_file)
     def show_about_window(self, widget):
-        #if not self.g_about_window:
-        if 1:
-            trans = _("SOLFEGETRANSLATORS")
-            if trans == 'SOLFEGETRANSLATORS':
-                trans = ""
-            pixbuf = self.render_icon('solfege-icon', gtk.ICON_SIZE_DIALOG)
-            self.g_about_window = gnome.ui.About("GNU Solfege",
-                  configureoutput.VERSION_STRING,
-                  copyright.decode("iso-8859-1"),
-                  """
-GNU Solfege is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
+        trans = _("SOLFEGETRANSLATORS")
+        if trans == 'SOLFEGETRANSLATORS':
+            trans = ""
+        pixbuf = self.render_icon('solfege-icon', gtk.ICON_SIZE_DIALOG)
+        a = self.g_about_window = gtk.AboutDialog()
+        a.set_logo(pixbuf)
+        a.set_name("GNU Solfege")
+        a.set_website("http://www.solfege.org")
+        a.set_version(configureoutput.VERSION_STRING)
+        a.set_copyright(copyright.decode('iso-8859-1') + """
 
-This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.""",                  ["Tom Cato Amundsen",
-                  'Oliver Vollmer %s' % _("(toolbar icons)"),
-                  'Giovanni Chierico %s' % _("(some lessonfiles)"),
-                  'Michael Becker %s' % _("(some lessonfiles)"),
-                  'Joe Lee %s' % _("(sound code for the MS Windows port)"),
-                  'Steve Lee %s' % _("(ported winmidi.c to gcc)"),
-                  'Thibaus Cousin %s' % _("(spec file for SuSE 8.2)"),
-                  'David Coe %s' %_("(spec file cleanup)"),
-                  'David Petrou %s' % _("(testing and portability fixes for FreeBSD)"),
-                  'Han-Wen Nienhuys %s' % _("(the music font from Lilypond)"),
-                  'Jan Nieuwenhuizen %s' % _("(the music font from Lilypond)"),
-                  ],
-                  ["Tom Cato Amundsen", "Tom Eykens"],
-                  trans, pixbuf)
-            #self.g_about_window.connect('destroy', self.hide_about_window)
-            #self.g_about_window.show()
-            #FIXME this sucks.
-            i = self.g_about_window.run()
-            #self.g_about_window.destroy()
-    def hide_about_window(self, widget):
+GNU Solfege is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU General Public License for more details.""")
+        a.set_authors(["Tom Cato Amundsen",
+              'Oliver Vollmer %s' % _("(toolbar icons)"),
+              'Giovanni Chierico %s' % _("(some lessonfiles)"),
+              'Michael Becker %s' % _("(some lessonfiles)"),
+              'Joe Lee %s' % _("(sound code for the MS Windows port)"),
+              'Steve Lee %s' % _("(ported winmidi.c to gcc)"),
+              'Thibaus Cousin %s' % _("(spec file for SuSE 8.2)"),
+              'David Coe %s' %_("(spec file cleanup)"),
+              'David Petrou %s' % _("(testing and portability fixes for FreeBSD)"),
+              'Han-Wen Nienhuys %s' % _("(the music font from Lilypond)"),
+              'Jan Nieuwenhuizen %s' % _("(the music font from Lilypond)"),
+              ])
+        a.set_documenters(["Tom Cato Amundsen",
+                "Tom Eykens",
+                ])
+        a.set_translator_credits(_("SOLFEGETRANSLATORS"))
+        self.g_about_window.run()
         self.g_about_window.destroy()
-        self.g_about_window = None
     def do_tree_editor(self, *v):
         # Raise it if it already exist.
         if self.g_learning_tree_editor:
@@ -555,7 +619,7 @@ This program is distributed in the hope that it will be useful, but WITHOUT ANY
             else:
                 self.box_dict[self.m_viewer].g_notebook.set_current_page(0)
         if self.box_dict[module].m_t.m_P:
-            if isinstance(self.box_dict[module].m_t, abstract.LessonbasedTeacher):
+            if isinstance(self.box_dict[module].m_t, abstract.Teacher):
                 self.set_title("Solfege - " + self.box_dict[module].m_t.m_P.header.title)
             else:
                 self.set_title("Solfege - " + self.box_dict[module].m_t.exercise_data['name'].replace("_", ""))
@@ -640,13 +704,6 @@ This program is distributed in the hope that it will be useful, but WITHOUT ANY
         # rebuild the menus in case we hav passed a test.
         self.create_practise_and_test_menu()
 
-if splash_win:
-    splash_win.show_progress("running gnome.program_init")
-
-gnome.program_init('solfege', configureoutput.VERSION_STRING)
-prog = gnome.program_get()
-prog.parse_args()
-
 import locale
 locale.setlocale(locale.LC_NUMERIC, "C")
 
@@ -658,7 +715,7 @@ cfg.set_bool('config/no_random', bool(options.no_random))
 
 if splash_win:
     splash_win.show_progress("creating LessonfileManager")
-lessonfile_manager = lessonfile.LessonFileManager()
+lessonfile_manager = lessonfile.LessonFileManager(options.debug)
 
 if splash_win:
     splash_win.hide()
@@ -678,6 +735,18 @@ def start_app(prefix, datadir):
     if splash_win:
         splash_win.destroy()
 
+    def ef(t, value, traceback):
+        if options.debug:
+            msg = "ehooked:" + str(value)
+        else:
+            msg = str(value)
+        if issubclass(t, lessonfile.LessonfileException):
+            w.display_error_message(msg, str(t))
+        elif issubclass(t, osutils.ExecutableDoesNotExist):
+            w.display_error_message(msg, str(t))
+        else:
+            sys.__excepthook__(t, msg, traceback)
+    sys.excepthook = ef
     import gtk
     gtk.main()
 
